@@ -15,6 +15,7 @@ from rest_framework.authtoken.models import Token
 from django_sso_app.core import app_settings
 
 from django_sso_app.core.apps.profiles.models import Profile
+from django_sso_app.core.apps.groups.models import Group
 
 User = get_user_model()
 
@@ -81,7 +82,7 @@ class TestStatus(TestCase):
             "date_joined": "2018-12-31T00:00:00+0000",
             "is_active": True,
             "email_verified": True,
-            "groups": [],
+            "groups": ['users'],
             "profile": {
                 "url": "http://localhost:8000/api/v1/auth/profiles/{}/".format(np_uuid),
                 "created_at": "2019-12-05T19:56:41+0000",
@@ -112,12 +113,14 @@ class TestStatus(TestCase):
             "email": "new_profile@example.com",
             "password": "pwd"
         }
+
         self.valid_new_profile_jwt_payload = {
                 'id':999,
                 'fingerprint': 'xxx',
                 'sso_id': np_uuid,
                 'sso_rev': 5
             }
+
         self.valid_new_profile_jwt = jwt.encode(
             self.valid_new_profile_jwt_payload,
             'secret',
@@ -167,7 +170,7 @@ class TestStatus(TestCase):
             )
             print('response', response.data)
 
-            self.assertEqual(response.status_code, status.HTTP_200_OK)
+            self.assertEqual(response.status_code, status.HTTP_200_OK, 'profile not created')
 
             self.assertEqual(Profile.objects.filter(sso_id=self.valid_new_profile['sso_id']).count(), 1,
                              'no profile model')
@@ -203,6 +206,10 @@ class TestStatus(TestCase):
             self.assertEqual(response.status_code, status.HTTP_200_OK)
             self.assertEqual(response.data.get('sso_id'), str(self.valid_new_profile['sso_id']), 'sso_id differs')
 
+            created_profile = Profile.objects.get(sso_id=self.valid_new_profile['sso_id'])
+            self.assertEqual(created_profile.groups.count(), 1, 'group not created')
+            self.assertEqual(created_profile.groups.filter(name='users').count(), 1, 'group not same')
+
         with self.settings(DJANGO_SSO_APP_SHAPE='app_apigateway',
                            DJANGO_SSO_APP_SERVICE_URL='http://example.com'):
             # deleting pre generated user
@@ -227,3 +234,7 @@ class TestStatus(TestCase):
 
             self.assertEqual(response.status_code, status.HTTP_200_OK)
             self.assertEqual(response.data.get('sso_id'), str(self.valid_new_profile['sso_id']), 'sso_id differs')
+
+            created_profile = Profile.objects.get(sso_id=self.valid_new_profile['sso_id'])
+            self.assertEqual(created_profile.groups.count(), 1, 'group not created')
+            self.assertEqual(created_profile.groups.filter(name='users').count(), 1, 'group not same')
