@@ -1,15 +1,21 @@
 import logging
 
 from django.contrib.auth import get_user_model
+from django import forms
 
 from allauth.account.adapter import get_adapter
 from allauth.account.models import EmailAddress
 from rest_framework import serializers
 
+from ...utils import set_session_key
 from ... import app_settings
 from ...permissions import is_staff
 from ...serializers import AbsoluteUrlSerializer
+from ..profiles.models import Profile
+from ..profiles.utils import update_profile
 from ..profiles.serializers import ProfileSerializer
+from ..services.models import Service
+from .utils import create_local_user_from_object
 
 User = get_user_model()
 
@@ -44,6 +50,57 @@ class UserRevisionSerializer(serializers.ModelSerializer):
         return instance.sso_rev
 
 
+class UserProfileSerializer(AbsoluteUrlSerializer):
+    sso_id = serializers.CharField(required=False)
+    sso_rev = serializers.IntegerField(required=False)
+
+    subscriptions = serializers.SerializerMethodField(required=False)
+
+    ssn = serializers.CharField(required='ssn' in app_settings.REQUIRED_PROFILE_FIELDS)
+    phone = serializers.CharField(required='phone' in app_settings.REQUIRED_PROFILE_FIELDS)
+
+    first_name = serializers.CharField(required='first_name' in app_settings.REQUIRED_PROFILE_FIELDS)
+    last_name = serializers.CharField(required='last_name' in app_settings.REQUIRED_PROFILE_FIELDS,
+                                      allow_null=True,
+                                      allow_blank=True)
+
+    description = serializers.CharField(required='description' in app_settings.REQUIRED_PROFILE_FIELDS,
+                                        allow_null=True, allow_blank=True)
+    picture = serializers.CharField(required='picture' in app_settings.REQUIRED_PROFILE_FIELDS, allow_null=True)
+    birthdate = serializers.DateField(required='birthdate' in app_settings.REQUIRED_PROFILE_FIELDS,
+                                      allow_null=True)
+
+    latitude = serializers.FloatField(required='latitude' in app_settings.REQUIRED_PROFILE_FIELDS)
+    longitude = serializers.FloatField(required='longitude' in app_settings.REQUIRED_PROFILE_FIELDS)
+
+    country = serializers.CharField(required='country' in app_settings.REQUIRED_PROFILE_FIELDS)
+    address = serializers.CharField(required='address' in app_settings.REQUIRED_PROFILE_FIELDS, allow_null=True)
+
+    language = serializers.CharField(required='language' in app_settings.REQUIRED_PROFILE_FIELDS,
+                                     allow_null=True)
+
+    class Meta:
+        model = Profile
+        read_only_fields = (
+            'url',
+            'created_at',
+            'sso_id', 'sso_rev',
+            'subscriptions',
+            'is_unsubscribed',
+            'groups')
+        fields = read_only_fields + app_settings.PROFILE_FIELDS
+
+    def get_subscriptions(self, instance):
+        serialized = []
+        if app_settings.BACKEND_ENABLED:
+            for el in instance.subscriptions.all():
+                serialized.append({
+                    'service_url': el.service.service_url,
+                    'is_unsubscribed': getattr(el, 'is_unsubscribed', False)
+                })
+        return serialized
+
+
 class UserSerializer(AbsoluteUrlSerializer):
     sso_id = serializers.SerializerMethodField(required=False)
     sso_rev = serializers.SerializerMethodField(required=False)
@@ -68,13 +125,14 @@ class UserSerializer(AbsoluteUrlSerializer):
     def get_sso_rev(self, instance):
         return instance.sso_rev
 
-    def get_email_verified(self, instance):
+    def get_email_verified(self, user):
         if app_settings.BACKEND_ENABLED:
             try:
-                email_address = instance.emailaddress_set.get(email=instance.email)
-                return email_address.verified
+                verified_email_addresses = user.emailaddress_set.filter(email=user.email, verified=True)
+                return verified_email_addresses.count() > 0
+
             except:
-                logger.info('{} has not verified email.'.format(instance))
+                logger.info('user "{}" has no verified emails'.format(user))
                 return False
 
         return True
@@ -101,59 +159,6 @@ class UserSerializer(AbsoluteUrlSerializer):
         # return the modified representation
         return ret
 
-    def create(self, validated_data):
-        request = self.context.get("request")
-        requesting_user = request.user
-        adapter = get_adapter(request)
-
-        requesting_user_is_staff = is_staff(requesting_user)
-        skip_confirmation = request.GET.get('skip_confirmation', None)
-        must_confirm_email = True
-        password_is_hashed = request.query_params.get('password_is_hashed', None)
-
-        new_email = validated_data.get('email', None)
-
-        if password_is_hashed:
-            new_password = validated_data.pop('password', None)
-        else:
-            new_password = validated_data.get('password', None)
-            self.validated_data.update(password1=new_password)
-            self.validated_data.update(password2=new_password)
-
-        assert (new_email is not None and new_password is not None)
-
-        logger.info(
-            'User "{}" wants to create new user "{}"'.format(requesting_user, validated_data))
-
-        if requesting_user_is_staff and skip_confirmation is not None:
-            must_confirm_email = False
-
-        new_user = adapter.save_user(request, User(), self, True)
-
-        created_email = EmailAddress.objects.add_email(request, new_user,
-                                                       new_email,
-                                                       confirm=must_confirm_email)
-
-        logger.info(
-            'A new EmailAddress for {0} has been created with id {1}'
-            ''.format(
-                new_email, created_email.id))
-
-        if must_confirm_email:
-            logger.info(
-                '{0} must confirm email {1}'.format(new_user, new_email))
-        else:
-            logger.info(
-                '{0} must NOT confirm email {1}'.format(new_user, new_email))
-            adapter.confirm_email(request, created_email)
-
-        if password_is_hashed:
-            new_user.password = new_password
-
-        new_user.save()
-
-        return new_user
-
     def update(self, instance, validated_data):
         user = instance
         request = self.context.get("request")
@@ -170,49 +175,52 @@ class UserSerializer(AbsoluteUrlSerializer):
         new_email = validated_data.get('email', None)
         new_password = validated_data.get('password', None)
 
-        logger.info(
-            'User {0} wants to update user {1}: {2}'.format(requesting_user, user, validated_data))
+        logger.info('User "{}" wants to update user "{}"'.format(requesting_user, user))
 
-        if (new_email is not None):
+        if (new_email is not None) and (new_email != user.email):
 
-            if new_email == user.email:  # !! because of app shitty logic (
-                # always sends all fields)
-                logger.info('{0} is has same email'.format(user))
+            logger.info('user "{}" is changing email for user "{}"'.format(requesting_user,
+                                                                           user))
 
-            else:
-                logger.info(
-                    '{0} is changing email for user {1}, email confirmation '
-                    'is {2}'.format(
-                        requesting_user, user, must_confirm_email))
+            restore_as_primary = False
 
+            try:
+                adapter.validate_unique_email(new_email)
+
+            except forms.ValidationError:
+                if EmailAddress.objects.filter(email__iexact=new_email).first() is not None:
+                    logger.info('User "{}" wants to reset email "{}" as primary'.format(requesting_user, new_email))
+                    restore_as_primary = True
+                    must_confirm_email = False
+                else:
+                    # email already taken
+                    raise
+
+            if not restore_as_primary:
                 created_email = EmailAddress.objects.add_email(request, user,
                                                                new_email,
                                                                confirm=must_confirm_email)
-                logger.info(
-                    'A new EmailAddress for {0} has been created with id {1}'
-                    ''.format(
-                        new_email, created_email.id))
 
-                if must_confirm_email:
-                    if requesting_user_is_staff:
-                        logger.info(
-                            'Unconfirming all emails for {0} while email has been updated by staff user {1}'.format(
-                                user,
-                                requesting_user))
+                logger.info('A new user EmailAddress for user "{}" has been created ({})'.format(user,
+                                                                                                 created_email.id))
 
-                        adapter.unconfirm_all_user_emails(user)
+            if must_confirm_email:
+                adapter.unconfirm_all_user_emails(user)
 
-                    logger.info(
-                        '{0} must confirm email {1}'.format(user, new_email))
-                else:
+                logger.info('"{}" must confirm email "{}"'.format(user, new_email))
+            else:
+                if not restore_as_primary:
                     adapter.confirm_email(None, created_email)
 
+            user.email = new_email  # set user model email
+
+            if not restore_as_primary:
                 setattr(user, '__django_sso_app__email_updated', True)
+                set_session_key(request, '__django_sso_app__user_email_updated', True)
 
         if (new_password is not None):
-            logger.info(
-                '{} is changing password for user {}'.format(requesting_user,
-                                                             user))
+            logger.info('user "{}" is changing password for user "{}"'.format(requesting_user,
+                                                                              user))
 
             if password_is_hashed:
                 logger.info('password is hashed')
@@ -223,6 +231,7 @@ class UserSerializer(AbsoluteUrlSerializer):
 
             setattr(user, '__django_sso_app__password_updated', True)
 
+        # updating user model
         user.save()
 
         return user
@@ -235,6 +244,57 @@ class UserSerializer(AbsoluteUrlSerializer):
         return self.validated_data
 
 
+class NewUserSerializer(UserSerializer):
+    profile = UserProfileSerializer(required=False)
+
+    def create(self, validated_data):
+        request = self.context.get('request')
+        requesting_user = request.user
+        adapter = get_adapter(request)
+
+        requesting_user_is_staff = is_staff(requesting_user)
+        skip_confirmation = request.GET.get('skip_confirmation', None)
+        must_confirm_email = True
+        password_is_hashed = request.query_params.get('password_is_hashed', None)
+
+        if password_is_hashed:
+            new_password = validated_data.pop('password', None)
+        else:
+            new_password = validated_data.get('password', None)
+            validated_data.update(password1=new_password)
+            validated_data.update(password2=new_password)
+
+        assert (new_password is not None)
+
+        logger.info('User "{}" wants to create new user "{}"'.format(requesting_user, validated_data))
+
+        if requesting_user_is_staff and skip_confirmation is not None:
+            must_confirm_email = False
+
+        new_user = create_local_user_from_object(validated_data, not must_confirm_email)
+
+        # setting new_user profile serializer field
+        setattr(new_user, 'profile', update_profile(new_user.sso_app_profile, validated_data.get('profile')))
+
+        if password_is_hashed:
+            new_user.password = new_password
+            new_user.save()
+
+        try:
+            new_user_subscriptions =  request.data.get('profile', {}).get('subscriptions', [])
+            if len(new_user_subscriptions) > 0:
+                for new_subscription in new_user_subscriptions:
+                    new_subscription_service_name = new_subscription.get('service').get('name')
+                    service = Service.objects.get(name=new_subscription_service_name)
+                    adapter.subscribe_profile_to_service(new_user.sso_app_profile, service)
+
+        except Exception:
+            logger.exception('Error subscribing service on user creation')
+            raise serializers.ValidationError('Error subscribing service')
+
+        return new_user
+
+
 class SuccessfullLoginResponseSerializer(serializers.Serializer):
     """
     Login success response serializer
