@@ -2,12 +2,12 @@ import os
 
 import cherrypy
 
-from django_sso_app.backend.config.wsgi import application
+from django_sso_app.config.wsgi import application
 
 
 def run(host="0.0.0.0", port=5000):
     # django settings
-    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'django_sso_app.backend.config.settings.local')
+    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'django_sso_app.config.settings.local')
 
     # Mount the application
     cherrypy.tree.graft(application, "/")
